/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.commands;

import org.eclipse.gef.commands.Command;

import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Connects two EclipseEclipseRelationElements with an EclipseEclipseRelationship.
 * @author Dennis Wagelaar
 * @version 0.2.0, 02/12/2003
 * @since 0.2.0
 */
public class ConnectionCommand extends Command
{

  /**
   * @uml property=sourceParticipant associationEnd={multiplicity={(0 1)}}
   */
  private RelationElement sourceParticipant = null;

  /**
   * @uml property=targetParticipant associationEnd={multiplicity={(0 1)}}
   */
  private RelationElement targetParticipant = null;

  /**
   * @uml property=oldSourceParticipant associationEnd={multiplicity={(0 1)}}
   */
  private RelationElement oldSourceParticipant = null;

  /**
   * @uml property=oldTargetParticipant associationEnd={multiplicity={(0 1)}}
   */
  private RelationElement oldTargetParticipant = null;

  /**
   * @uml property=relationship associationEnd={multiplicity={(0 1)}}
   */
  private Relationship relationship = null;

  private int sourceIndex = -1;
  private int targetIndex = -1;
  private int oldSourceIndex = -1;
  private int oldTargetIndex = -1;

  /**
   * Creates a ConnectionCommand.
   */
  public ConnectionCommand() 
  {
    super("Connect");
  }

  /**
   * @see org.eclipse.gef.commands.Command#execute()
   */
  public void execute() 
  {
    primExecute();
  }
  
  /**
   * @see org.eclipse.gef.commands.Command#redo()
   */
  public void redo()
  {
    primExecute();
  }
  
  /**
   * @see org.eclipse.gef.commands.Command#undo()
   */
  public void undo()
  {
    if (sourceParticipant != null) 
    {
      sourceParticipant.removeRelationship(relationship);
      relationship.setSourceParticipant(oldSourceParticipant);
      if (oldSourceParticipant != null)
        oldSourceParticipant.addRelationship(oldSourceIndex, relationship);
    }
    if (targetParticipant != null) 
    {
      targetParticipant.removeRelationship(relationship);
      relationship.setTargetParticipant(oldTargetParticipant);
      if (oldTargetParticipant != null)
        oldTargetParticipant.addRelationship(oldTargetIndex, relationship);
    }
  }

  /**
   * Gets the source participant.
   * @return the source participant.
   * @uml property=sourceParticipant
   */
  public RelationElement getSourceParticipant()
  {
    return sourceParticipant;
  }

  /**
   * Sets the source participant.
   * @param sourceParticipant - the source participant.
   * @uml property=sourceParticipant
   */
  public void setSourceParticipant(RelationElement sourceParticipant)
  {
    this.sourceParticipant = sourceParticipant;
  }

  /**
   * Gets the target participant.
   * @return the target participant.
   * @uml property=targetParticipant
   */
  public RelationElement getTargetParticipant()
  {
    return targetParticipant;
  }

  /**
   * Sets the target participant.
   * @param targetParticipant - the target participant.
   * @uml property=targetParticipant
   */
  public void setTargetParticipant(RelationElement targetParticipant)
  {
    this.targetParticipant = targetParticipant;
  }

  /**
   * Gets the relationship.
   * @return the relationship.
   * @uml property=relationship
   */
  public Relationship getRelationship()
  {
    return relationship;
  }

  /**
   * Sets the relationship.
   * @param relationship - the relationship.
   * @uml property=relationship
   */
  public void setRelationship(Relationship relationship)
  {
    this.relationship = relationship;
    if (relationship != null)
    {
      this.oldSourceParticipant =
        (RelationElement) relationship.getSourceParticipant();
      this.oldTargetParticipant =
        (RelationElement) relationship.getTargetParticipant();
      if (oldSourceParticipant != null)
        this.oldSourceIndex =
          oldSourceParticipant.getRelationships().indexOf(relationship);
      if (oldTargetParticipant != null)
        this.oldTargetIndex =
          oldTargetParticipant.getRelationships().indexOf(relationship);
    }
  }

  /**
   * Gets the source participant relationship index.
   * @return the source participant relationship index.
   * @uml property=sourceIndex
   */
  public int getSourceIndex()
  {
    return sourceIndex;
  }

  /**
   * Sets the source participant relationship index.
   * @param sourceIndex - the source participant relationship index.
   * @uml property=sourceIndex
   */
  public void setSourceIndex(int sourceIndex)
  {
    this.sourceIndex = sourceIndex;
  }

  /**
   * Gets the target participant relationship index.
   * @return the target participant relationship index.
   * @uml property=targetIndex
   */
  public int getTargetIndex()
  {
    return targetIndex;
  }

  /**
   * Sets the target participant relationship index.
   * @param targetIndex - the target participant relationship index.
   * @uml property=targetIndex
   */
  public void setTargetIndex(int targetIndex)
  {
    this.targetIndex = targetIndex;
  }

  /**
   * Execute implementation.
   * @see org.eclipse.gef.commands.Command#execute()
   */
  protected void primExecute() 
  {
    if (sourceParticipant != null) 
    {
      if (oldSourceParticipant != null)
        oldSourceParticipant.removeRelationship(relationship);
      relationship.setSourceParticipant(sourceParticipant);
      sourceParticipant.addRelationship(sourceIndex, relationship);
    }
    if (targetParticipant != null) 
    {
      if (oldTargetParticipant != null)
        oldTargetParticipant.removeRelationship(relationship);
      relationship.setTargetParticipant(targetParticipant);
      targetParticipant.addRelationship(targetIndex, relationship);
    }
  }
}
