/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.editparts;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import be.ac.vub.cocompose.eclipse.editpolicies.CoComposeNodeEditPolicy;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.RelationElement;

/**
 * Eclipse editpart controller for EclipseRelationElement objects.
 * @author Dennis Wagelaar
 * @version 0.2.0, 02/10/2003
 * @since 0.2.0
 */
public abstract class RelationElementEditPart extends NamespaceEditPart implements NodeEditPart
{

  /**
   * @uml property=anchor associationEnd={multiplicity={(1 1)}}
   */
  private ConnectionAnchor anchor = new ChopboxAnchor(getFigure());

  
  /**
   * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
   */
  public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection)
  {
    return anchor;
  }

  /**
   * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
   */
  public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection)
  {
    return anchor;
  }

  /**
   * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
   */
  public ConnectionAnchor getSourceConnectionAnchor(Request request)
  {
    return anchor;
  }

  /**
   * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
   */
  public ConnectionAnchor getTargetConnectionAnchor(Request request)
  {
    return anchor;
  }

  /**
   * Handles changes in properties of this. It is 
   * activated through the PropertyChangeListener.
   * It updates children, source and target connections,
   * and the visuals of this based on the property
   * changed.
   * @param evt  Event which details the property change.
   * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
   */
  public void propertyChange(PropertyChangeEvent evt)
  {
    String id = evt.getPropertyName();
    if (id.equals(Properties.ID_POSITION) ||
        id.equals(Properties.ID_BGCOLOR)) 
      refreshVisuals();
    else if (Properties.ID_SOURCERELATIONSHIPS.equals(id))
      refreshSourceConnections();
    else if (Properties.ID_TARGETRELATIONSHIPS.equals(id))
      refreshTargetConnections();
    else
      super.propertyChange(evt);
  }

  /**
   * Installs edit policies.
   * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
   */
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new CoComposeNodeEditPolicy());
  }

  /**
   * Gets the model associated with this.
   * @return the model associated with this.
   */
  protected RelationElement getRelationElement()
  {
    return (RelationElement) getModel();
  }

  /**
   * Returns a list of connections for which this is the 
   * source.
   * @return List of connections.
   * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
   */
  protected List getModelSourceConnections()
  {
    return getRelationElement().getSourceRelationships();
  }

  /**
   * Returns a list of connections for which this is the 
   * target.
   * @return  List of connections.
   * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
   */
  protected List getModelTargetConnections()
  {
    return getRelationElement().getTargetRelationships();
  }

  /**
   * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
   */
  protected void refreshVisuals() 
  {
    java.awt.Color cl = getRelationElement().getBackgroundColor();
    getFigure().setBackgroundColor(new Color(Display.getCurrent(),
                                             cl.getRed(),
                                             cl.getGreen(),
                                             cl.getBlue()));
    Point pos = new Point(getRelationElement().getPosition().x,
                          getRelationElement().getPosition().y);
    Rectangle rect = new Rectangle(pos, new Dimension(-1, -1));
    ((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), rect);
    super.refreshVisuals();
  }
}
