/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import be.ac.vub.cocompose.lang.datatypes.Multiplicity;

/**
 * Eclipse Multiplicity property source.
 * @author Dennis Wagelaar
 * @version 0.2.0, 12/12/2003
 * @since 0.2.0
 */
public class MultiplicityPropertySource implements IPropertySource
{
  public static final String ID_LOWER = "lowerBound";
  public static final String ID_UPPER = "upperBound";

  private static IPropertyDescriptor[] descriptors;

  /**
   * @uml property=multiplicity associationEnd={multiplicity={(1 1)}}
   */
  private Multiplicity multiplicity = null;

  static{
    descriptors = new IPropertyDescriptor[] {
      new TextPropertyDescriptor(ID_LOWER, "lower bound"),
      new TextPropertyDescriptor(ID_UPPER, "upper bound")
    };
  }

  /**
   * Creates a MultiplicityPropertySource.
   * @param multiplicity - the subject multiplicity.
   */
  public MultiplicityPropertySource(Multiplicity multiplicity)
  {
    super();
    this.multiplicity = new Multiplicity(multiplicity);
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return this;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    return descriptors;
  }

  /**
   * Gets the value of the given property.
   * @param id - the property id.
   * @return the property value or null.
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    if (ID_LOWER.equals(id))
    {
      return new String(new Integer(multiplicity.getLower()).toString());
    }
    if (ID_UPPER.equals(id))
    {
      return new String(new Integer(multiplicity.getUpper()).toString());
    }
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return true;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
  }

  /**
   * Sets the value of the given property.
   * @param id - the property id.
   * @param value - the new value for the property.
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    if (ID_LOWER.equals(id))
    {
      try
      {
        Integer lower = new Integer((String) value);
        multiplicity.setLower(lower.intValue());
      }
      catch (NumberFormatException e)
      { // set to *
        multiplicity.setLower(Multiplicity.MANY);
      }
    }
    if (ID_UPPER.equals(id)) 
    {
      try
      {
        Integer upper = new Integer((String) value);
        multiplicity.setUpper(upper.intValue());
      }
      catch (NumberFormatException e)
      { // set to *
        multiplicity.setUpper(Multiplicity.MANY);
      }
    }
  }

  /**
   * Gets the value of this property source.
   * @return the property value.
   */
  public Multiplicity getValue()
  {
    return new Multiplicity(multiplicity);
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    if (multiplicity != null)
      return multiplicity.toString();
    else
      return super.toString();
  }
}
