/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.model.datatypes;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Role;

/**
 * Eclipse property source for the Concept part-of relationship.
 * @author Dennis Wagelaar
 * @version 0.2.4, 21/01/2004
 * @since 0.2.0
 */
public class PartOfPropertySource implements IPropertySource
{
  /**
   * @uml property=concept associationEnd={multiplicity={(0 1)}}
   */
  private Concept concept = null;

  /**
   * @uml property=model associationEnd={multiplicity={(0 1)}}
   */
  private Namespace model = null;

  private List roles = null;

  /**
   * @uml property=desc associationEnd={multiplicity={(0 -1)} elementType=org.eclipse.ui.views.properties.IPropertyDescriptor}
   * 
   */
  private IPropertyDescriptor[] desc = null;

  /**
   * @uml property=values associationEnd={multiplicity={(0 -1)} elementType=java.lang.String}
   * 
   */
  private List values = null;

  
  /**
   * Creates a PartOfPropertySource
   * @param concept the Concept for which to edit the PartOf relationship.
   */
  public PartOfPropertySource(Concept concept)
  {
    this.concept = concept;
    Namespace ns = concept.getNamespace();
    while (ns.getNamespace() != null)
      ns = ns.getNamespace();
    this.model = ns;
    this.roles = model.getDeepOwnedElements(Role.class);
    this.values = createValues();
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
   */
  public Object getEditableValue()
  {
    return this;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
   */
  public IPropertyDescriptor[] getPropertyDescriptors()
  {
    try
    {
      if (desc == null)
      {
        desc = createDescriptorsArray(createPropertyDescriptors());
      }
    }
    catch (Exception e)
    { // do nothing
      e.printStackTrace();
    }
    return desc;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
   */
  public Object getPropertyValue(Object id)
  {
    List partOf = getPartOf();
    try
    {
      int index = Integer.parseInt((String) id);
      if (index < partOf.size())
      {
        return new Integer(values.indexOf(getFullName((Role) partOf.get(index))));
      }
      else
        return new Integer(values.indexOf("<<none>>"));
    }
    catch (NumberFormatException e)
    { // do nothing
    }
    return null;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
   */
  public boolean isPropertySet(Object id)
  {
    return true;
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
   */
  public void resetPropertyValue(Object id)
  {
    List partOf = getPartOf();
    try
    {
      int index = Integer.parseInt((String) id);
      if (index < partOf.size())
      {
        concept.removePartOf((Role) partOf.get(index));
      }
    }
    catch (NumberFormatException e)
    { // do nothing
      e.printStackTrace();
    }
  }

  /**
   * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
   */
  public void setPropertyValue(Object id, Object value)
  {
    List partOf = getPartOf();
    try
    {
      int index = Integer.parseInt((String) id);
      int valueIndex = ((Integer) value).intValue();
      if (index < partOf.size())
      {
        concept.removePartOf((Role) partOf.get(index));
        if (valueIndex > 0)
        {
          concept.addPartOf(index, (Role) roles.get(valueIndex - 1));
        }
      }
      else
      {
        if (valueIndex > 0)
        {
          concept.addPartOf((Role) roles.get(valueIndex - 1));
        }
      }
    }
    catch (NumberFormatException e)
    { // do nothing
      e.printStackTrace();
    }
  }
 
  /**
   * Gets the PartOf members.
   * @return the PartOf list.
   */
  public List getPartOf()
  {
    return concept.getPartOf();  
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    String value = "[";
    List partOf = getPartOf();
    for (int index = 0; index < partOf.size(); index++)
    {
      value += partOf.get(index);
      if (index < partOf.size() - 1)
        value += ", ";
    }
    value += "]";
    return value;
  }

  /**
   * Creates the property descriptors list
   * @return the property descriptors list
   */
  private List createPropertyDescriptors()
  {
    List partOf = getPartOf();
    List descs = new Vector();
    String[] values = createStringArray(this.values);
    Iterator po = partOf.iterator();
    while (po.hasNext())
    {
      Role role = (Role) po.next();
      Integer id = new Integer(partOf.indexOf(role));
      ComboBoxPropertyDescriptor desc = new ComboBoxPropertyDescriptor(id.toString(), id.toString(), values);
      desc.setLabelProvider(new IndexedLabelProvider(roles));
      descs.add(desc);
    }
    Integer id = new Integer(partOf.size());
    ComboBoxPropertyDescriptor desc = new ComboBoxPropertyDescriptor(id.toString(), id.toString(), values);
    desc.setLabelProvider(new IndexedLabelProvider(roles));
    descs.add(desc);
    return descs;
  }

  /**
   * Creates the available values (roles).
   * @return the list of available values.
   */
  private List createValues()
  {
    List values = new Vector();
    values.add("<<none>>");
    Iterator roles = this.roles.iterator();
    while (roles.hasNext())
    {
      Role role = (Role) roles.next();
      values.add(getFullName(role));
    }
    return values;
  }

  /**
   * Creates the property descriptors array from a Collection.
   * @param desc - the descriptors collection.
   * @return the property descriptors array.
   */
  private IPropertyDescriptor[] createDescriptorsArray(Collection desc)
  {
    IPropertyDescriptor[] idesc = new IPropertyDescriptor[desc.size()];
    int index = 0;
    Iterator descIt = desc.iterator();
    while (descIt.hasNext())
    {
      idesc[index] = (IPropertyDescriptor) descIt.next();
      index++;
    }
    return idesc;
  }

  /**
   * Creates a string array from a Collection.
   * @param coll - the collection.
   * @return the string array.
   */
  private String[] createStringArray(Collection coll)
  {
    String[] strings = new String[coll.size()];
    int index = 0;
    Iterator collIt = coll.iterator();
    while (collIt.hasNext())
    {
      strings[index] = (String) collIt.next();
      index++;
    }
    return strings;
  }
  
  /**
   * Returns the full model-wide unique name of the given element
   * @param element the element to return the name for
   * @return the full element name
   */
  private String getFullName(ModelElement element)
  {
    String value = element.getName();
    Namespace ns = element.getNamespace();
    while (ns.getNamespace() != null)
    {
      value = ns.getName() + "." + value;
      ns = ns.getNamespace();
    }
    return value;
  }
}
