/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.net.MalformedURLException;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.ModelElementException;

/**
 * Introduces methods for writing to URIs.
 * @author Dennis Wagelaar 
 * @version 0.2.4, 21/01/2004
 * @since 0.2.0
 */
abstract public class FileModelWriter implements ModelWriter
{

  /**
   * @supplierRole streamWriter
   * @link aggregation
   * @associates <{StreamWriter}>
   * @uml property=streamWriter associationEnd={multiplicity={(0 1)}}
   */
  private StreamWriter streamWriter = null;


  /**
   * Writes the model to its URI.
   * @param model - the model to write.
   * @exception IOException - if the model could not be written. 
   * @exception ModelElementException - if a model element could not be written.
   */
  public void write(Model model) throws IOException, ModelElementException 
  {
    if (getStreamWriter() == null)
      throw new IOException("streamWriter is null");
    getStreamWriter().setOutputStream(getOutputStream(model.getUri()));
  }

  /**
   * Retrieves the file corresponding to the given URI.
   * @param uri - the URI to look at.
   * @return the file.
   * @exception IOException - if the file could not be retrieved.
   */
  private File getFile(String uri) throws IOException 
  {
    try
    { // if no protocol specified, assume this is a file path
      if (uri.indexOf(':') == -1)
        return new File(uri).getAbsoluteFile();
      // else parse URI
      URL url = new URL(uri);
      if (!url.getProtocol().equals("file"))
        throw new IOException("Can only write to \"file:\" URIs");
      return new File(url.getFile());
    }
    catch (MalformedURLException e)
    {
      throw new IOException(e.getMessage());
    }
  }

  /**
   * Opens an output stream to the given URI.
   * @param uri - the URI to look at.
   * @return the output stream.
   * @exception IOException - if the output stream could not be opened.
   */
  protected OutputStream getOutputStream(String uri) throws IOException 
  {
    return new FileOutputStream(getFile(uri));
  }

  /**
   * Gets the stream writer.
   * @return the stream writer.
   * @uml property=streamWriter
   */
  public StreamWriter getStreamWriter()
  {
    return streamWriter;
  }

  /**
   * Sets the stream writer.
   * @param streamWriter - the stream writer.
   * @uml property=streamWriter
   */
  public void setStreamWriter(StreamWriter streamWriter)
  {
    this.streamWriter = streamWriter;
  }

}
