/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.xmi;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;
import be.ac.vub.cocompose.lang.ModelVisitor;

/**
 * Retrieves the XMI attributes for each model element.
 * @author Dennis Wagelaar 
 * @version 0.2.3, 20/01/2004
 * @since 0.2.0
 */
public class XMIElementAttributesWriter extends DefaultSuperModelVisitor 
{

  /**
   * @uml property=attributes associationEnd={multiplicity={(1 1)}}
   */
  private AttributesImpl attributes = new AttributesImpl();

  /**
   * @supplierRole type
   * @link aggregationByValue
   * @supplierCardinality 1
   * @uml property=type associationEnd={multiplicity={(1 1)}}
   */
  private XMIElementType type = new XMIElementType();

  /**
   * @uml property=rootAttributes associationEnd={multiplicity={(0 1)}}
   */
  private Attributes rootAttributes = null;


  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
    super.visitModelElement(modelElement);
    addRootAttributes();
    type.visit(modelElement);
    attributes.addAttribute(XMI.NS_URI, "type", XMI.qName("type"), "CDATA", type.getType());
    attributes.addAttribute(XMI.NS_URI, "id", XMI.qName("id"), "ID", modelElement.getId());
    if (modelElement.getName() != null)
      attributes.addAttribute(XMI.CoCompose.NS_URI, "name", XMI.CoCompose.qName("name"), "CDATA", modelElement.getName());
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    super.visitRefinedElement(refinedElement);
    if (refinedElement.getIconUri() != null)
      attributes.addAttribute(XMI.CoCompose.NS_URI, "iconUri", XMI.CoCompose.qName("iconUri"), "CDATA", refinedElement.getIconUri());
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    super.visitModel(model);
    if (attributes.getIndex(XMI.CoCompose.qName("name")) > -1)
      attributes.removeAttribute(attributes.getIndex(XMI.CoCompose.qName("name")));
    attributes.removeAttribute(attributes.getIndex(XMI.qName("type")));
    if (model.getDefaultConcept() != null)
      attributes.addAttribute(XMI.CoCompose.NS_URI, "defaultConcept", XMI.CoCompose.qName("defaultConcept"), "CDATA", model.getDefaultConcept().getId());
  }

  /**
   * Visits a Constraint.
   * @param constraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitConstraint(Constraint constraint) throws ModelElementException
  {
    super.visitConstraint(constraint);
    if (constraint.isInverted())
      attributes.addAttribute(XMI.CoCompose.NS_URI, "inverted", XMI.CoCompose.qName("inverted"), "CDATA", "true");
    else
      attributes.addAttribute(XMI.CoCompose.NS_URI, "inverted", XMI.CoCompose.qName("inverted"), "CDATA", "false");
  }

  /**
   * Visits a Multiplicity.
   * @param multiplicity - the element to visit.
   * @exception ModelElementException
   */
  public void visitMultiplicity(Multiplicity multiplicity) throws ModelElementException
  {
    super.visitMultiplicity(multiplicity);
    attributes.addAttribute(XMI.CoCompose.NS_URI, "lower", XMI.CoCompose.qName("lower"), 
      "CDATA", Integer.toString(multiplicity.getLower()));
    attributes.addAttribute(XMI.CoCompose.NS_URI, "upper", XMI.CoCompose.qName("upper"), 
      "CDATA", Integer.toString(multiplicity.getUpper()));
  }

  /**
   * Gets the XMI attributes for the visited element.
   * @return the XMI attributes.
   */
  public Attributes getAttributes()
  {
    return attributes;
  }

  /**
   * Starts the visitor.
   * @param visitedElement - the element to start visiting at.
   * @exception ModelElementException
   */
  public void visit(VisitedElement visitedElement) throws ModelElementException
  {
    attributes.clear();
    visitedElement.accept(current);
  }

  /**
   * Gets the root attributes to add to the first element.
   * @return the root attributes.
   * @uml property=rootAttributes
   */
  public Attributes getRootAttributes()
  {
    return rootAttributes;
  }

  /**
   * Sets the root attributes to add to the first element.
   * @param rootAttributes - the root attributes.
   * @uml property=rootAttributes
   */
  public void setRootAttributes(Attributes rootAttributes)
  {
    this.rootAttributes = rootAttributes;
  }

  /**
   * Adds the root attributes to the current attributes. Note that these only
   * need to be added once for a document and only to the root element.
   */
  private void addRootAttributes()
  {
    rootAttributes = getRootAttributes();
    if (rootAttributes != null)
    {
      for (int index = 0; index < rootAttributes.getLength(); index ++)
      {
        attributes.addAttribute(
          rootAttributes.getURI(index),
          rootAttributes.getLocalName(index),
          rootAttributes.getQName(index),
          rootAttributes.getType(index),
          rootAttributes.getValue(index));
      }
      setRootAttributes(null);
    }
  }

  /**
   * Sets a stacked visitor for this and the contained visitors.
   * @param stacked - the stacked visitor.
   */
  public void setStacked(ModelVisitor stacked)
  {
    super.setStacked(stacked);
    type.setStacked(stacked);
  }
}
