/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.xmi;

import java.util.Hashtable;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;

/** 
 * Reads the links in a CoCompose model from an XMI file (requires first pass by
 * XMIModelReader). 
 * @author Dennis Wagelaar 
 * @version 0.2.3, 20/01/2004
 * @since 0.2.0
 */
public class XMIModelLinkReader extends DefaultHandler 
{

  /**
   * @uml property=model associationEnd={multiplicity={(0 1)}}
   */
  private Model model = null;

  /**
   * @uml property=elementMap associationEnd={multiplicity={(0 1)} qualifier=(ref:java.lang.String
   * value:be.ac.vub.cocompose.lang.core.RefinedElement)}
   */
  private Hashtable elementMap = new Hashtable();

  /**
   * @uml property=namespace associationEnd={multiplicity={(0 1)}}
   */
  private Namespace namespace = null;

  /**
   * @uml property=element associationEnd={multiplicity={(0 1)}}
   */
  private ModelElement element = null;

  /**
   * @uml property=relationship associationEnd={multiplicity={(0 1)}}
   */
  private Relationship relationship = null;

  /**
   * @uml property=role associationEnd={multiplicity={(0 1)}}
   */
  private Role role = null;

  /**
   * @uml property=concept associationEnd={multiplicity={(0 1)}}
   */
  private Concept concept = null;

  /**
   * @uml property=elementReader associationEnd={multiplicity={(0 1)}}
   */
  private XMIElementLinkAttributesReader elementReader = null;

  /**
   * Creates an XMIModelLinkReader
   * @param model the model after one pass by XMIModelReader
   * @param elementMap the map of id -&gt; element
   */
  public XMIModelLinkReader(Model model, Hashtable elementMap)
    throws ModelElementException
  {
    if (model == null)
      throw new ModelElementException("Model may not be null");
    if (elementMap == null)
      throw new ModelElementException("ElementMap may not be null");
    this.model = model;
    this.namespace = model;
    this.elementMap = elementMap;
    elementReader = new XMIElementLinkAttributesReader(elementMap);
  }

  //
  // ContentHandler methods
  //

  /**
   * Processing instruction.
   *
   * @param target Processing instruction element name.
   * @param data Processing instruction element data.
   */
  public void processingInstruction(String target, String data)
    throws SAXException
  {
  } // processingInstruction

  /**
   * Start document.
   */
  public void startDocument() throws SAXException
  {
  } //startDocument

  /**
   * Start element.
   * @param uri XML namespace URI.
   * @param local local element name.
   * @param raw raw (qualified) element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    System.out.println("start " + uri + local + raw);
    if (uri.equals(XMI.NS_URI))
      startXMIElement(local, attrs);
    else
      startCoComposeElement(local, attrs);
  } //startElement

  /**
   * Start XMI element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startXMIElement(String local, Attributes attrs) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startCoComposeElement(String local, Attributes attrs) throws SAXException
  {
    try
    {
      if (local.equals("Model"))
      {
        elementReader.setAttributes(attrs);
        elementReader.visit(model);
      }
      else if (local.equals("ownedElement"))
      {
        startOwnedElement(attrs);
      }
      else if (local.equals("participant"))
      {
        startParticipant(attrs);
      }
      else if (local.equals("partOf"))
      {
        startPartOf(attrs);
      }
      else if (local.equals("part"))
      {
        startPart(attrs);
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startOwnedElement(Attributes attrs) throws ModelElementException
  {
    String id = attrs.getValue(XMI.qName("id"));
    element = (ModelElement) elementMap.get(id);
    if (element instanceof Namespace)
      namespace = (Namespace) element;
    parseOwnedElement();
    // TODO debug println
    System.out.println("Namespace now is " + namespace);
  }

  /**
   * Parses a participant tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startParticipant(Attributes attrs) throws ModelElementException
  {
    if (relationship == null)
      throw new ModelElementException("not within relationship tag while parsing a participant tag");
    String ref = attrs.getValue(XMI.qName("idref"));
    if (ref == null)
      throw new ModelElementException("no idref found in participant tag");
    RelationElement participant = (RelationElement) elementMap.get(ref);
    if (participant == null)
      throw new ModelElementException("participant not found");
    if (relationship.getSourceParticipant() == null)
    {
      relationship.setSourceParticipant(participant);
    }
    else if (relationship.getTargetParticipant() == null)
    {
      relationship.setTargetParticipant(participant);
    }
    else
    {
      throw new ModelElementException("more than two participant tags found within a relationship tag");
    }
    participant.addRelationship(relationship);
  }

  /**
   * Parses a partOf tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startPartOf(Attributes attrs) throws ModelElementException
  {
    if (concept == null)
      throw new ModelElementException("not within a Concept while parsing a partOf tag");
    String ref = attrs.getValue(XMI.qName("idref"));
    if (ref == null)
      throw new ModelElementException("no idref found in partOf tag");
    Role partOf = (Role) elementMap.get(ref);
    if (partOf == null)
      throw new ModelElementException("partOf not found");
    concept.addPartOf(partOf);
  }

  /**
   * Parses a part tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startPart(Attributes attrs) throws ModelElementException
  {
    if (role == null)
      throw new ModelElementException("not within a Role while parsing a part tag");
    String ref = attrs.getValue(XMI.qName("idref"));
    if (ref == null)
      throw new ModelElementException("no idref found in part tag");
    Concept part = (Concept) elementMap.get(ref);
    if (part == null)
      throw new ModelElementException("part not found");
    role.addPart(part);
  }

  /**
   * Characters.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char[] ch, int start, int length)
    throws SAXException
  {
  } //characters

  /**
   * Ignorable whitespace.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
    throws SAXException
  {
  } //ignorableWhitespace

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    if (uri.equals(XMI.NS_URI))
      endXMIElement(local);
    else
      endCoComposeElement(local);
  } //endElement

  /**
   * End XMI element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endXMIElement(String local) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endCoComposeElement(String local) throws SAXException
  {
    try
    {
      if (local.equals("ownedElement"))
      {
        endOwnedElement();
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @throws ModelElementException
   */
  public void endOwnedElement() throws ModelElementException
  {
    if ((element instanceof Namespace) && (namespace != null))
    {
      namespace = namespace.getNamespace();
    }
    element = namespace;
    parseOwnedElement();
    // TODO debug println
    System.out.println("Namespace re-set to " + namespace);
  }

  //
  // DTDHandler methods
  //

  /**
   * DTD declaration.
   *
   * @param name DTD name.
   * @param publicId DTD public ID.
   * @param systemId DTD system ID.
   */
  public void notationDecl(String name, String publicId, String systemId)
    throws SAXException
  {
  } //notationDecl
  
  /**
   * Parses the current ownedElement according to its class and sets
   * the corresponding attributes.
   */
  private void parseOwnedElement()
  {
    relationship = null;
    role = null;
    concept = null;
    if (element == null)
      return;
    if (element instanceof Relationship)
      relationship = (Relationship) element;
    else if (element instanceof Role)
      role = (Role) element;
    else if (element instanceof Concept)
      concept = (Concept) element;
  }
}
