/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.xmi;

import java.awt.Color;
import java.awt.Point;
import java.io.IOException;
import java.util.Hashtable;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.io.xml.XML;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;

/** 
 * Reads a CoCompose model from an XMI file (requires second pass by XMIModelLinkReader). 
 * @author Dennis Wagelaar 
 * @version 0.2.2, 16/01/2004
 * @since 0.2.0
 */
public class XMIModelReader extends DefaultHandler implements ModelReader 
{

  /**
   * @uml property=model associationEnd={multiplicity={(0 1)}}
   */
  private Model model = null;

  private String uri = null;

  /**
   * @uml property=elementMap associationEnd={multiplicity={(0 1)} qualifier=(getId:java.lang.String
   * element:be.ac.vub.cocompose.lang.core.ModelElement)}
   */
  private Hashtable elementMap = new Hashtable();

  /**
   * @uml property=elementReader associationEnd={multiplicity={(0 1)}}
   */
  private XMIElementAttributesReader elementReader = null;

  /**
   * @uml property=typeSwitch associationEnd={multiplicity={(0 1)}}
   */
  private XMIElementTypeSwitch typeSwitch = null;

  /**
   * @uml property=linkReader associationEnd={multiplicity={(0 1)}}
   */
  private XMIModelLinkReader linkReader = null;

  /**
   * @uml property=namespace associationEnd={multiplicity={(0 1)}}
   */
  private Namespace namespace = null;

  /**
   * @uml property=element associationEnd={multiplicity={(0 1)}}
   */
  private ModelElement element = null;
  
  /**
   * @uml property=relationship associationEnd={multiplicity={(0 1)}}
   */
  private Relationship relationship = null;

  
  /**
   * Reads the model from its uri.
   * @param uri - the uri to read from.
   * @param factory - the model factory used to create the elements.
   * @return the model
   * @exception IOException - if the model could not be read.
   * @exception ModelElementException - if a model element could not be read.
   */
  public Model read(String uri, ModelFactory factory) throws IOException, ModelElementException
  {
    uri = uri.replace('\\', '/');
    this.uri = uri;
    String name = uri.substring(uri.lastIndexOf('/') + 1, uri.lastIndexOf('.'));
    elementReader = new XMIElementAttributesReader();
    typeSwitch = new XMIElementTypeSwitch(factory);
    model = factory.createModel();
    namespace = model;
    try
    {
      model.setName(name);
      model.setUri(uri);
      // create XML parser
      XMLReader xmlParser = (XMLReader) Class.forName(XML.PARSER).newInstance();
      xmlParser.setFeature(XML.VALIDATION, XML.SetValidation);
      xmlParser.setFeature(XML.NAMESPACES, XML.SetNamespaces);
      xmlParser.setFeature(XML.SCHEMA, XML.SetSchema);
      xmlParser.setFeature(XML.SCHEMA_FULL, XML.SetSchemaFull);
      // start parser for 1st pass
      xmlParser.setContentHandler(this);
      xmlParser.parse(uri);
      // start parser for 2nd pass
      linkReader = new XMIModelLinkReader(model, elementMap);
      xmlParser.setContentHandler(linkReader);
      xmlParser.parse(uri);
      return model;
    }
    catch (Exception e)
    {
      throw new ModelElementException(e);
    }
  }

  //
  // ContentHandler methods
  //

  /**
   * Processing instruction.
   *
   * @param target Processing instruction element name.
   * @param data Processing instruction element data.
   */
  public void processingInstruction(String target, String data)
    throws SAXException
  {
  } // processingInstruction

  /**
   * Start document.
   */
  public void startDocument() throws SAXException
  {
  } //startDocument

  /**
   * Start element.
   * @param uri XML namespace URI.
   * @param local local element name.
   * @param raw raw (qualified) element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    System.out.println("start " + uri + local + raw);
    if (uri.equals(XMI.NS_URI))
      startXMIElement(local, attrs);
    else
      startCoComposeElement(local, attrs);
  } //startElement

  /**
   * Start XMI element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startXMIElement(String local, Attributes attrs) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startCoComposeElement(String local, Attributes attrs) throws SAXException
  {
    try
    {
      if (local.equals("Model"))
      {
        elementReader.setAttributes(attrs);
        elementReader.visit(model);
        elementMap.put(model.getId(), model);
      }
      else if (local.equals("ownedElement"))
      {
        startOwnedElement(attrs);
      }
      else if (local.equals("position"))
      {
        startPosition(attrs);
      }
      else if (local.equals("color"))
      {
        startColor(attrs);
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startOwnedElement(Attributes attrs) throws ModelElementException
  {
    String type = attrs.getValue(XMI.qName("type"));
    element = typeSwitch.createElement(type);
    if (element == null)
      throw new ModelElementException("element type " + type + " not recognized");
    elementReader.setAttributes(attrs);
    elementReader.visit(element);
    if (namespace == null)
      throw new ModelElementException("Namespace is null for " + type + " " + element.getName());
    namespace.addOwnedElement(element);
    elementMap.put(element.getId(), element);
    if (element instanceof Namespace)
      namespace = (Namespace) element;
    else if (element instanceof Relationship)
      relationship = (Relationship) element;
    // TODO debug println
    System.out.println("Namespace now is " + namespace);
  }

  /**
   * Finalises an ownedElement tag
   * @throws ModelElementException
   */
  public void endOwnedElement() throws ModelElementException
  {
    if ((element instanceof Namespace) && (namespace != null))
      namespace = namespace.getNamespace();
    // TODO debug println
    System.out.println("Namespace re-set to " + namespace);
  }

  /**
   * Parses a position tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startPosition(Attributes attrs) throws ModelElementException
  {
    if (!(namespace instanceof RelationElement))
      throw new ModelElementException("Cannot find relation element to set position for");
    RelationElement element = (RelationElement) namespace;
    try
    {
      Point pos = new Point();
      pos.x = Integer.parseInt(attrs.getValue(XMI.Diagram.qName("x")));
      pos.y = Integer.parseInt(attrs.getValue(XMI.Diagram.qName("y")));
      element.setPosition(pos);
    }
    catch (NumberFormatException e)
    {
      throw new ModelElementException(e);
    }
  }

  /**
   * Parses a color tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startColor(Attributes attrs) throws ModelElementException
  {
    if (!(namespace instanceof RelationElement))
      throw new ModelElementException("Cannot find relation element to set color for");
    RelationElement element = (RelationElement) namespace;
    try
    {
      Color col = new Color(
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("r"))),
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("g"))),
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("b"))));
      element.setBackgroundColor(col);
    }
    catch (NumberFormatException e)
    {
      throw new ModelElementException(e);
    }
  }
  
  /**
   * Characters.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char[] ch, int start, int length)
    throws SAXException
  {
  } //characters

  /**
   * Ignorable whitespace.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
    throws SAXException
  {
  } //ignorableWhitespace

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    System.out.println("end " + uri + local + raw);
    if (uri.equals(XMI.NS_URI))
      endXMIElement(local);
    else
      endCoComposeElement(local);
  } //endElement

  /**
   * End XMI element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endXMIElement(String local) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endCoComposeElement(String local) throws SAXException
  {
    try
    {
      if (local.equals("ownedElement"))
      {
        endOwnedElement();
      }
    }
    catch (ModelElementException e)
    {
      throw new SAXException(e.getMessage());
    }
  }

  //
  // DTDHandler methods
  //

  /**
   * DTD declaration.
   *
   * @param name DTD name.
   * @param publicId DTD public ID.
   * @param systemId DTD system ID.
   */
  public void notationDecl(String name, String publicId, String systemId)
    throws SAXException
  {
  } //notationDecl
}
