/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.xml;

import java.io.*;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import be.ac.vub.cocompose.io.StreamWriter;

/**
 * XML writer for writing any XML document to an OutputStream. Based on the
 * SAX2 DefaultHandler class.
 * @author Dennis Wagelaar
 * @version 0.2.0, 09/01/2004
 * @since 0.1.0
 */

public class SAXWriter extends DefaultHandler implements StreamWriter
{
  // indent counter
  private int indent = 0;
  // last element cache
  private String lastElement = null;

  /**
   * Print writer for output.
   */
  private PrintWriter out;
  private String charset = "UTF-8";

  /**
   * Processing instruction.
   *
   * @param target Processing instruction element name.
   * @param data Processing instruction element data.
   */
  public void processingInstruction(String target, String data)
  {
    out.println('>');
    printIndent();
    out.print("<?" + target);
    if (data != null && data.length() > 0)
    {
      out.print(' ' + data);
    }
    out.print('?');
    out.flush();
  }

  /**
   * Start document.
   */
  public void startDocument()
  {
    out.print("<?xml version=\"1.0\" encoding=\"" + getCharset() + "\"?");
    out.flush();
  }

  /**
   * Start element.
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   * @param attrs Element attributes.
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs)
  {
    out.println('>');
    printIndent();
    out.print('<' + raw);
    printAttributes(attrs);
    out.flush();
    indent++;
    lastElement = raw;
  }

  /**
   * Characters.
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char[] ch, int start, int length)
  {
    if (lastElement != null)
    {
      out.println('>');
      printIndent();
      lastElement = null;
    }
    out.print(normalize(new String(ch, start, length)));
    out.flush();
  } //characters

  /**
   * Ignorable whitespace.
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
  {
    characters(ch, start, length);
  } //ignorableWhitespace

  /**
   * End element.
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   */
  public void endElement(String uri, String local, String raw)
  {
    indent--;
    if (lastElement != null)
    {
      if (lastElement.equals(raw))
      {
        out.print('/');
      }
      else
      {
        out.println('>');
        printIndent();
        out.print("</" + raw);
      }
    }
    else
    {
      out.println();
      printIndent();
      out.print("</" + raw);
    }
    lastElement = raw;
    out.flush();
  } //endElement


  /**
   * End document.
   */
  public void endDocument()
  {
    indent--;
    if (lastElement != null)
    {
      out.println('>');
      out.flush();
    }
  }

  /**
   * DTD declaration.
   *
   * @param name DTD name.
   * @param publicId DTD public ID.
   * @param systemId DTD system ID.
   */
  public void notationDecl(String name, String publicId, String systemId)
  {
    out.println('>');
    printIndent();
    out.print("<!DOCTYPE " + name);
    if (publicId != null)
    {
      out.print(" PUBLIC \"" + publicId + "\"");
    }
    if (systemId != null)
    {
      out.print(" SYSTEM \"" + systemId + "\"");
    }
    out.flush();
  }

  /**
   * Gets the encoding character set (e.g. "UTF-8").
   * @return the character set.
   * @uml property=charset
   */
  public String getCharset()
  {
    return charset;
  }

  /**
   * Sets the encoding character set (e.g. "UTF-8").
   * @param charset - the character set.
   * @exception UnsupportedEncodingException - if the character set is not supported.
   * @uml property=charset
   */
  public void setCharset(String charset) throws UnsupportedEncodingException
  { // test character set (java 1.4 allows for a nicer solution)
    new OutputStreamWriter(new ByteArrayOutputStream(), charset);
    this.charset = charset;
  }

  /**
   * Sets the output stream for writing.
   * @param outputStream - the output stream.
   */
  public void setOutputStream(OutputStream outputStream)
  {
    try
    {
      out = new PrintWriter(new OutputStreamWriter(outputStream, getCharset()));
    }
    catch (UnsupportedEncodingException e)
    {
    }
  }

  /**
   * Prints the required indentation. 
   */
  private void printIndent()
  {
    for (int i = 0; i < indent; i++)
    {
      out.print("  ");
    }
  }

  /**
   * Normalizes the given string (removes xml control chars).
   * @param s The string to normalize.
   */
  private String normalize(String s)
  {
    StringBuffer str = new StringBuffer();
    int len = (s != null) ? s.length() : 0;
    for (int i = 0; i < len; i++)
    {
      char ch = s.charAt(i);
      switch (ch)
      {
        case '<':
          str.append("&lt;");
          break;
        case '>':
          str.append("&gt;");
          break;
        case '&':
          str.append("&amp;");
          break;
        case '"':
          str.append("&quot;");
          break;
        case '\r':
        case '\n':
          str.append("&#");
          str.append(Integer.toString(ch));
          str.append(';');
          break;
        default:
        {
          str.append(ch);
        }
      } //switch
    } //for
    return str.toString();
  }

  /**
   * Prints the given attributes.
   * @param attrs - the attributes to print.
   */
  private void printAttributes(Attributes attrs)
  {
    if (attrs != null)
    {
      int len = attrs.getLength();
      for (int i = 0; i < len; i++)
      {
        printAttribute(attrs.getQName(i), attrs.getValue(i));
      }
    }
  }

  /**
   * Prints the given attribute name and value.
   * @param name - the attribute name.
   * @param value - the attribute value.
   */
  private void printAttribute(String name, String value)
  {
    out.print(' ' + name + "=\"" + normalize(value) + '"');
  }
} //class SAXWriter
