/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang;

import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Refinement;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.constraints.CompositeConstraint;
import be.ac.vub.cocompose.lang.constraints.AndConstraint;
import be.ac.vub.cocompose.lang.constraints.OrConstraint;
import be.ac.vub.cocompose.lang.properties.Property;
import be.ac.vub.cocompose.lang.properties.SimpleProperty;
import be.ac.vub.cocompose.lang.properties.ModelProperty;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;

/**
 * Interface for visiting a Model.
 * @author Dennis Wagelaar 
 * @version 0.2.0, 09/01/2004
 * @since 0.1.0
 */
public interface ModelVisitor
{
  /**
   * @supplierRole stacked
   * @directed 
   */
  /*# ModelVisitor lnkModelVisitor; */

  /**
   * Starts the visitor.
   * @exception ModelElementException
   * @param element - the element to start visiting at. 
   */
  void visit(VisitedElement element) throws ModelElementException;

  /**
   * Visits a ModelElement.
   * @exception ModelElementException
   * @param modelElement - the element to visit.
   */
  void visitModelElement(ModelElement modelElement) throws ModelElementException;

  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  void visitNamespace(Namespace namespace) throws ModelElementException;

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  void visitModel(Model model) throws ModelElementException;

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException 
   */
  void visitRelationship(Relationship relationship) throws ModelElementException;

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException 
   */
  void visitRelationElement(RelationElement relationElement) throws ModelElementException;

  /**
   * Visits a Refinement.
   * @param refinement - the element to visit.
   * @exception ModelElementException
   */
  void visitRefinement(Refinement refinement) throws ModelElementException;

  /**
   * Visits a SolutionPattern.
   * @param solutionPattern - the element to visit.
   * @exception ModelElementException
   */
  void visitSolutionPattern(SolutionPattern solutionPattern) throws ModelElementException;

  /**
   * Visits an ImplementationGenerator.
   * @param implementationGenerator - the element to visit.
   * @exception ModelElementException
   */
  void visitImplementationGenerator(ImplementationGenerator implementationGenerator) throws ModelElementException;

  /**
   * Visits an ImplementationRolePart.
   * @param rolePart - the element to visit.
   * @exception ModelElementException
   */
  void visitImplementationRolePart(ImplementationRolePart rolePart) throws ModelElementException;

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException;

  /**
   * Visits a RefinedElement.
   * @exception ModelElementException
   * @param refinedElement - the element to visit.
   */
  void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException;

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  void visitConcept(Concept concept) throws ModelElementException;

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  void visitRole(Role role) throws ModelElementException;

  /**
   * Visits a Constraint.
   * @param constraint - the element to visit.
   * @exception ModelElementException
   */
  void visitConstraint(Constraint constraint) throws ModelElementException;

  /**
   * Visits an ElementConstraint.
   * @param elementConstraint - the element to visit.
   * @exception ModelElementException
   */
  void visitElementConstraint(ElementConstraint elementConstraint) throws ModelElementException;

  /**
   * Visits a CompositeConstraint.
   * @param compositeConstraint - the element to visit.
   * @exception ModelElementException
   */
  void visitCompositeConstraint(CompositeConstraint compositeConstraint) throws ModelElementException;

  /**
   * Visits an AndConstraint.
   * @param andConstraint - the element to visit.
   * @exception ModelElementException
   */
  void visitAndConstraint(AndConstraint andConstraint) throws ModelElementException;

  /**
   * Visits an OrConstraint.
   * @param orConstraint - the element to visit.
   * @exception ModelElementException
   */
  void visitOrConstraint(OrConstraint orConstraint) throws ModelElementException;

  /**
   * Visits a Property.
   * @param property - the element to visit.
   * @exception ModelElementException
   */
  void visitProperty(Property property) throws ModelElementException;

  /**
   * Visits a SimpleProperty.
   * @param simpleProperty - the element to visit.
   * @exception ModelElementException
   */
  void visitSimpleProperty(SimpleProperty simpleProperty) throws ModelElementException;

  /**
   * Visits a ModelProperty.
   * @param modelProperty - the element to visit.
   * @exception ModelElementException
   */
  void visitModelProperty(ModelProperty modelProperty) throws ModelElementException;

  /**
   * Visits a Multiplicity.
   * @param multiplicity - the element to visit.
   * @exception ModelElementException 
   */
  void visitMultiplicity(Multiplicity multiplicity) throws ModelElementException;

  /**
   * Gets the stacked visitor.
   * @return the stacked visitor.
   * @uml property=stacked associationEnd={multiplicity={(0 1)} inverse={stacked:be.ac.vub.cocompose.lang.ModelVisitor}
   * }
   */
  ModelVisitor getStacked();

  /**
   * Sets a stacked visitor.
   * @param stacked - the stacked visitor.
   * @uml property=stacked
   */
  void setStacked(ModelVisitor stacked);

}
