package be.ac.vub.cocompose.lang.impl.constraints;

import java.util.Collection;
import java.util.Vector;
import java.util.Iterator;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.ModelVisitor;

/**
 * Model element constraint.
 * @author Dennis Wagelaar 
 * @version 0.2.0
 * @since 0.2.0
 */
public class DefaultElementConstraint extends DefaultConstraint implements ElementConstraint
{

  /**
   * @supplierRole constrainedElement
   * @supplierCardinality 1..*
   * @associates <{be.ac.vub.cocompose.lang.impl.DefaultModelElement}>
   * @uml property=constrainedElements associationEnd={multiplicity={(0 -1)} elementType=
   * be.ac.vub.cocompose.lang.core.ModelElement}
   */
  private Vector constrainedElements = new Vector();

  /**
   * Returns the constrained elements.
   * @return a copy of the collection of constrained elements.
   */
  public Collection getConstrainedElements()
  {
    return (Collection) constrainedElements.clone();
  }

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.*/
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitElementConstraint(this);
  }

  /**
   * Invokes superclass accept().
   * @param modelVisitor The visitor to accept.
   * @see #accept(ModelVisitor)*/
  public void acceptSuper(ModelVisitor modelVisitor) throws ModelElementException
  {
    super.accept(modelVisitor);
  }

  /**
   * Adds a constrained element.
   * @param constrainedElement - the element to add. 
   */
  public void addConstrainedElement(ModelElement constrainedElement)
  {
    constrainedElements.add(constrainedElement);
  }

  /**
   * Removes a constrained element.
   * @param constrainedElement - the element to remove. 
   */
  public void removeConstrainedElement(ModelElement constrainedElement)
  {
    constrainedElements.remove(constrainedElement);
  }

  /**
   * Checks whether the constraint holds.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  protected void concreteCheck() throws ModelElementException
  {
    Iterator constrElts = getConstrainedElements().iterator();
    while (constrElts.hasNext())
    {
      checkElement((ModelElement) constrElts.next(), this);
    }
  }

  /**
   * Check the constraint for one ModelElement.
   * @param element - the element to check.
   * @param root - the property container root to check against.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  private void checkElement(ModelElement element, Namespace root) throws ModelElementException 
  {
    if (element instanceof Namespace)
    {
      checkNamespace((Namespace) element, root);
      return;
    }
    else
    {
      if (root.getOwnedElements().size() != 0)
        throw new ModelElementException("Constraint " + getName() + " violated",
          this, ModelElementException.ERROR);
    }
  }

  /**
   * Check the constraint for one Namespace.
   * @param namespace - the namespace to check.
   * @param root - the property container root to check against.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  private void checkNamespace(Namespace namespace, Namespace root) throws ModelElementException 
  {
    Iterator ownedElts = root.getOwnedElements().iterator();
    while (ownedElts.hasNext())
    {
      ModelElement elt = (ModelElement) ownedElts.next();
      if (!namespace.isRegistered(elt.getName()))
        throw new ModelElementException("Constraint " + getName() + " violated",
          this, ModelElementException.ERROR);
      if (elt instanceof Namespace)
        checkElement(namespace.getOwnedElement(elt.getName()), (Namespace) elt);
    }
  }
}
