package be.ac.vub.cocompose.lang.impl.core;

import java.awt.Color;
import java.awt.Point;
import java.util.List;
import java.util.Vector;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Represents CoCompose language elements that can participate in a relationship.
 * @version 0.2.5, 03/03/2004
 * @since 0.2.0
 * @author Dennis Wagelaar 
 */
abstract public class DefaultRelationElement extends DefaultNamespace implements RelationElement
{
  private static final Color DEFAULT_COLOR = new Color(255, 255, 192);

  /**
   * @clientRole participant
   * @clientCardinality 2
   * @supplierRole relationship
   * @supplierCardinality 0..* {ordered}
   * @associates <{be.ac.vub.cocompose.lang.impl.core.DefaultRelationship}>
   * @undirected
   * @uml property=relationships associationEnd={multiplicity={(0 -1)} elementType=be.ac.vub.cocompose.lang.core.Relationship}
   * 
   */
  private Vector relationships = new Vector();

  /**
   * @uml property=sourceRelationships associationEnd={multiplicity={(0 -1)} elementType=
   * be.ac.vub.cocompose.lang.core.Relationship}
   */
  private Vector sourceRelationships = new Vector();

  /**
   * @uml property=targetRelationships associationEnd={multiplicity={(0 -1)} elementType=
   * be.ac.vub.cocompose.lang.core.Relationship}
   */
  private Vector targetRelationships = new Vector();

  private Point position = new Point(0,0);
  private Color bgcolor = getDefaultBackgroundColor();

  /**
   * Gets the default background color.
   * @return the default background color.
   */
  protected Color getDefaultBackgroundColor()
  {
    return DEFAULT_COLOR;
  }

  /**
   * Gets the relationships for this element.
   * @return a copy of the list of relationships.
   */
  public List getRelationships()
  {
    return (List) relationships;
  }

  /**
   * Gets the source relationships for this element.
   * @return the list of source relationships.
   */
  public List getSourceRelationships()
  {
    return (List) sourceRelationships;
  }

  /**
   * Gets the target relationships for this element.
   * @return the list of target relationships.
   */
  public List getTargetRelationships()
  {
    return (List) targetRelationships;
  }

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitRelationElement(this);
  }

  /**
   * Adds a relationship for this element.
   * @param relationship - the relationship to add.
   */
  public void addRelationship(Relationship relationship)
  {
    addRelationship(-1, relationship);
  }

  /**
   * Adds a relationship for this element at the specified index.
   * @param index - the index at which to add the relationship.
   * @param relationship - the relationship to add.
   */
  public void addRelationship(int index, Relationship relationship)
  {
    if (index > -1)
      relationships.add(index, relationship);
    else
      relationships.add(relationship);
    fireStructureChange(Properties.ID_RELATIONSHIPS, relationship);
    if (relationship.getSourceParticipant().equals(this))
      addSourceRelationship(translateIndex(index, relationships, sourceRelationships), relationship);
    else
      addTargetRelationship(translateIndex(index, relationships, targetRelationships), relationship);
  }

  /**
   * Removes a relationship from this element.
   * @param relationship - the relationship to remove. 
   */
  public void removeRelationship(Relationship relationship)
  {
    relationships.remove(relationship);
    fireStructureChange(Properties.ID_RELATIONSHIPS, relationship);
    if (relationship.getSourceParticipant().equals(this))
      removeSourceRelationship(relationship);
    else
      removeTargetRelationship(relationship);
  }

  /**
   * Gets the element position.
   * @return the position.
   * @uml property=position
   */
  public Point getPosition()
  {
    return position;
  }

  /**
   * Sets the element position.
   * @param position - the position.
   * @uml property=position
   */
  public void setPosition(Point position)
  {
    if (this.position.equals(position))
      return;
    this.position = position;
    firePropertyChange(Properties.ID_POSITION, null, position);
  }

  /**
   * Gets the background color.
   * @return the background color.
   */
  public Color getBackgroundColor()
  {
    return bgcolor;
  }

  /**
   * Sets the background color.
   * @param bgcolor - the background color.
   */
  public void setBackgroundColor(Color bgcolor)
  {
    if (this.bgcolor.equals(bgcolor))
      return;
    this.bgcolor = bgcolor;
    fireBackgroundColorUpdate();
  }
  
  /**
   * Fires an ID_BGCOLOR event for the hierarchy of owned elements.
   */
  protected void fireBackgroundColorUpdate()
  {
    firePropertyChange(Properties.ID_BGCOLOR, null, getBackgroundColor());
    List children = getOwnedElements();
    for (int i = 0; i < children.size(); i++)
    {
      if (children.get(i) instanceof DefaultRelationElement)
      {
        DefaultRelationElement child = (DefaultRelationElement) children.get(i);
        child.fireBackgroundColorUpdate();
      }
    }
  }

  /**
   * Adds a source relationship for this element at the specified index.
   * @param index - the index at which to add the relationship.
   * @param relationship - the relationship to add.
   */
  private void addSourceRelationship(int index, Relationship relationship)
  {
    if (index > -1)
      sourceRelationships.add(index, relationship);
    else
      sourceRelationships.add(relationship);
    fireStructureChange(Properties.ID_SOURCERELATIONSHIPS, relationship);
  }

  /**
   * Removes a source relationship from this element.
   * @param relationship - the relationship to remove. 
   */
  private void removeSourceRelationship(Relationship relationship)
  {
    sourceRelationships.remove(relationship);
    fireStructureChange(Properties.ID_SOURCERELATIONSHIPS, relationship);
  }

  /**
   * Adds a target relationship for this element at the specified index.
   * @param index - the index at which to add the relationship.
   * @param relationship - the relationship to add.
   */
  private void addTargetRelationship(int index, Relationship relationship)
  {
    if (index > -1)
      targetRelationships.add(index, relationship);
    else
      targetRelationships.add(relationship);
    fireStructureChange(Properties.ID_TARGETRELATIONSHIPS, relationship);
  }

  /**
   * Removes a target relationship from this element.
   * @param relationship - the relationship to remove. 
   */
  private void removeTargetRelationship(Relationship relationship)
  {
    targetRelationships.remove(relationship);
    fireStructureChange(Properties.ID_TARGETRELATIONSHIPS, relationship);
  }
}
