/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.operations;

import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Undo for a DeleteElement operation.
 * @author Dennis Wagelaar 
 * @version 0.2.4, 22/01/2004
 * @since 0.2.0
 * @see DeleteElement
 */
public class UndeleteElement extends DefaultSuperModelVisitor 
{

  /**
   * @uml property=deletedElement associationEnd={multiplicity={(0 1)}}
   */
  private ModelElement deletedElement = null;

  private int elementIndex = -1;
  private int sourceIndex = -1;
  private int targetIndex = -1;

  /**
   * Gets the deleted element to undelete.
   * @return the deleted element.
   * @uml property=deletedElement
   */
  public ModelElement getDeletedElement()
  {
    return deletedElement;
  }

  /**
   * Sets the deleted element to undelete.
   * @param deletedElement - the deleted element.
   * @uml property=deletedElement
   */
  public void setDeletedElement(ModelElement deletedElement)
  {
    this.deletedElement = deletedElement;
  }


  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception <{be.ac.vub.cocompose.lang.ModelElementException}>
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
    super.visitModelElement(modelElement);
    if (modelElement.getNamespace() != null)
      modelElement.getNamespace().addOwnedElement(getElementIndex(), modelElement);
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception <{ModelElementException}>
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    super.visitRelationship(relationship);
    relationship.getSourceParticipant().addRelationship(getSourceIndex(), relationship);
    relationship.getTargetParticipant().addRelationship(getTargetIndex(), relationship);
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    super.visitConcept(concept);
    if (concept.getDefaultConceptOf() != null)
    {
      concept.getDefaultConceptOf().setDefaultConcept(concept);
    }
  }

  /**
   * Gets the element participant index at which a relationship was deleted.
   * @return the index.
   * @uml property=elementIndex
   */
  public int getElementIndex()
  {
    return elementIndex;
  }

  /**
   * Sets the element participant index at which a relationship was deleted.
   * @param elementIndex - the index.
   * @uml property=elementIndex
   */
  public void setElementIndex(int elementIndex)
  {
    this.elementIndex = elementIndex;
  }

  /**
   * Gets the source participant index at which a relationship was deleted.
   * @return the index.
   * @uml property=sourceIndex
   */
  public int getSourceIndex()
  {
    return sourceIndex;
  }

  /**
   * Sets the source participant index at which a relationship was deleted.
   * @param sourceIndex - the index.
   * @uml property=sourceIndex
   */
  public void setSourceIndex(int sourceIndex)
  {
    this.sourceIndex = sourceIndex;
  }

  /**
   * Gets the target participant index at which a relationship was deleted.
   * @return the index.
   * @uml property=targetIndex
   */
  public int getTargetIndex()
  {
    return targetIndex;
  }

  /**
   * Sets the target participant index at which a relationship was deleted.
   * @param targetIndex - the index.
   * @uml property=targetIndex
   */
  public void setTargetIndex(int targetIndex)
  {
    this.targetIndex = targetIndex;
  }
}
